@@ -3,7 +3,7 @@
 //
 // Code available from: https://verilator.org
 //
-// Copyright 2001-2022 by Wilson Snyder. This program is free software; you
+// Copyright 2001-2023 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -29,7 +29,6 @@
 
 #include <algorithm>
 #include <cerrno>
-#include <ctime>
 #include <fcntl.h>
 
 #if defined(_WIN32) && !defined(__MINGW32__) && !defined(__CYGWIN__)
@@ -62,16 +61,6 @@ constexpr unsigned VL_TRACE_MAX_VCD_CODE_SIZE = 5;  // Maximum length of a VCD s
 // cache-lines.
 constexpr unsigned VL_TRACE_SUFFIX_ENTRY_SIZE = 8;  // Size of a suffix entry
 
-//=============================================================================
-// Utility functions: TODO: put these in a common place and share them.
-
-template <size_t N>
-static size_t roundUpToMultipleOf(size_t value) {
-    static_assert((N & (N - 1)) == 0, "'N' must be a power of 2");
-    size_t mask = N - 1;
-    return (value + mask) & ~mask;
-}
-
 //=============================================================================
 // Specialization of the generics for this trace format
 
@@ -230,12 +219,10 @@ VerilatedVcd::~VerilatedVcd() {
     if (m_wrBufp) VL_DO_CLEAR(delete[] m_wrBufp, m_wrBufp = nullptr);
     deleteNameMap();
     if (m_filep && m_fileNewed) VL_DO_CLEAR(delete m_filep, m_filep = nullptr);
-#ifdef VL_THREADED
     if (parallel()) {
         assert(m_numBuffers == m_freeBuffers.size());
         for (auto& pair : m_freeBuffers) VL_DO_CLEAR(delete[] pair.first, pair.first = nullptr);
     }
-#endif
 }
 
 void VerilatedVcd::closePrev() {
@@ -364,17 +351,13 @@ void VerilatedVcd::printIndent(int level_change) {
 
 void VerilatedVcd::dumpHeader() {
     printStr("$version Generated by VerilatedVcd $end\n");
-    printStr("$date ");
-    {
-        const time_t tick = time(nullptr);
-        tm ticktm;
-        VL_LOCALTIME_R(&tick, &ticktm);
-        constexpr size_t LEN_BUF = 50;
-        char buf[LEN_BUF];
-        std::strftime(buf, LEN_BUF, "%c", &ticktm);
-        printStr(buf);
-    }
-    printStr(" $end\n");
+
+    // Verilator used to put in a $date here.  Although $date is shown in
+    // IEEE examples, and it is common in VCD writers, VCD readers don't
+    // seem to care about it.  Thus, we omit the $date so artifacts are
+    // more likely to be reproducible.  If use cases show up that require
+    // the $date command to be present, it could be re-added with support
+    // for the SOURCE_DATE_EPOCH hook.
 
     printStr("$timescale ");
     printStr(timeResStr().c_str());  // lintok-begin-on-ref
@@ -427,28 +410,14 @@ void VerilatedVcd::dumpHeader() {
             if (*np == ' ') np++;
             if (*np == '\t') break;  // tab means signal name starts
             printIndent(1);
-            // Find character after name end
-            const char* sp = np;
-            while (*sp && *sp != ' ' && *sp != '\t' && !(*sp & '\x80')) sp++;
-
-            printStr("$scope ");
-            if (*sp & '\x80') {
-                switch (*sp & 0x7f) {
-                case VLT_TRACE_SCOPE_STRUCT: printStr("struct "); break;
-                case VLT_TRACE_SCOPE_INTERFACE: printStr("interface "); break;
-                case VLT_TRACE_SCOPE_UNION: printStr("union "); break;
-                default: printStr("module ");
-                }
-            } else {
-                printStr("module ");
-            }
+            printStr("$scope module ");
 
             for (; *np && *np != ' ' && *np != '\t'; np++) {
                 if (*np == '[') {
                     printStr("[");
                 } else if (*np == ']') {
                     printStr("]");
-                } else if (!(*np & '\x80')) {
+                } else {
                     *m_writep++ = *np;
                 }
             }
@@ -482,7 +451,7 @@ void VerilatedVcd::declare(uint32_t code, const char* name, const char* wirep, b
         m_suffixes.resize(nextCode() * VL_TRACE_SUFFIX_ENTRY_SIZE * 2, 0);
     }
 
-    // Keep upper bound on bytes a single signal cna emit into the buffer
+    // Keep upper bound on bytes a single signal can emit into the buffer
     m_maxSignalBytes = std::max<size_t>(m_maxSignalBytes, bits + 32);
     // Make sure write buffer is large enough, plus header
     bufferResize(m_maxSignalBytes + 1024);
@@ -534,7 +503,7 @@ void VerilatedVcd::declare(uint32_t code, const char* name, const char* wirep, b
                 std::strlen(buf));  // Code (overwrite separator if isBit)
     entryp[length + !isBit] = '\n';  // Replace '\0' with line termination '\n'
     // Set length of suffix (used to increment write pointer)
-    entryp[VL_TRACE_SUFFIX_ENTRY_SIZE - 1] = !isBit + length + 1;
+    entryp[VL_TRACE_SUFFIX_ENTRY_SIZE - 1] = static_cast<char>(length + !isBit + 1);
     decl += " ";
     decl += basename;
     if (array) {
@@ -550,6 +519,9 @@ void VerilatedVcd::declare(uint32_t code, const char* name, const char* wirep, b
     m_namemapp->emplace(hiername, decl);
 }
 
+void VerilatedVcd::declEvent(uint32_t code, const char* name, bool array, int arraynum) {
+    declare(code, name, "event", array, arraynum, false, false, 0, 0);
+}
 void VerilatedVcd::declBit(uint32_t code, const char* name, bool array, int arraynum) {
     declare(code, name, "wire", array, arraynum, false, false, 0, 0);
 }
@@ -574,9 +546,8 @@ void VerilatedVcd::declDouble(uint32_t code, const char* name, bool array, int a
 
 VerilatedVcd::Buffer* VerilatedVcd::getTraceBuffer() {
     VerilatedVcd::Buffer* const bufp = new Buffer{*this};
-#ifdef VL_THREADED
     if (parallel()) {
-        // Note: This is called from VeriltedVcd::dump, which already holds the lock
+        // Note: This is called from VerilatedVcd::dump, which already holds the lock
         // If no buffer available, allocate a new one
         if (m_freeBuffers.empty()) {
             constexpr size_t pageSize = 4096;
@@ -593,15 +564,13 @@ VerilatedVcd::Buffer* VerilatedVcd::getTraceBuffer() {
         bufp->m_size = pair.second;
         bufp->adjustGrowp();
     }
-#endif
     // Return the buffer
     return bufp;
 }
 
 void VerilatedVcd::commitTraceBuffer(VerilatedVcd::Buffer* bufp) {
     if (parallel()) {
-#if VL_THREADED
-        // Note: This is called from VeriltedVcd::dump, which already holds the lock
+        // Note: This is called from VerilatedVcd::dump, which already holds the lock
         // Resize output buffer. Note, we use the full size of the trace buffer, as
         // this is a lot more stable than the actual occupancy of the trace buffer.
         // This helps us to avoid re-allocations due to small size changes.
@@ -616,9 +585,6 @@ void VerilatedVcd::commitTraceBuffer(VerilatedVcd::Buffer* bufp) {
         bufferCheck();
         // Put buffer back on free list
         m_freeBuffers.emplace_back(bufp->m_bufp, bufp->m_size);
-#else
-        VL_FATAL_MT(__FILE__, __LINE__, "", "Unreachable");
-#endif
     } else {
         // Needs adjusting for emitTimeChange
         m_writep = bufp->m_writep;
@@ -665,7 +631,6 @@ void VerilatedVcdBuffer::finishLine(uint32_t code, char* writep) {
     m_writep = writep + suffixp[VL_TRACE_SUFFIX_ENTRY_SIZE - 1];
 
     if (m_owner.parallel()) {
-#ifdef VL_THREADED
         // Double the size of the buffer if necessary
         if (VL_UNLIKELY(m_writep >= m_growp)) {
             // Compute occupied size of current buffer
@@ -685,9 +650,6 @@ void VerilatedVcdBuffer::finishLine(uint32_t code, char* writep) {
             // Adjust resize limit
             adjustGrowp();
         }
-#else
-        VL_FATAL_MT(__FILE__, __LINE__, "", "Unreachable");
-#endif
     } else {
         // Flush the write buffer if there's not enough space left for new information
         // We only call this once per vector, so we need enough slop for a very wide "b###" line
@@ -706,6 +668,19 @@ void VerilatedVcdBuffer::finishLine(uint32_t code, char* writep) {
 // verilated_trace_imp.h, which is included in this file at the top),
 // so always inline them.
 
+VL_ATTR_ALWINLINE
+void VerilatedVcdBuffer::emitEvent(uint32_t code, VlEvent newval) {
+    const bool triggered = newval.isTriggered();
+    // TODO : It seems that untriggered events are not filtered
+    // should be tested before this last step
+    if (triggered) {
+        // Don't prefetch suffix as it's a bit too late;
+        char* wp = m_writep;
+        *wp++ = '1';
+        finishLine(code, wp);
+    }
+}
+
 VL_ATTR_ALWINLINE
 void VerilatedVcdBuffer::emitBit(uint32_t code, CData newval) {
     // Don't prefetch suffix as it's a bit too late;
